@@ -31,6 +31,12 @@
 #define warningcall Rf_warningcall
 void warningcall (SEXP call, char *format, ...);
 
+#define errorcall Rf_errorcall
+void errorcall (SEXP call, char *format, ...);
+
+/* global variable */
+int hdf5_global_verbosity;
+
 static herr_t
 ref_string (hid_t sid, hid_t did, H5T_cdata_t *cdata,
 	    size_t count, size_t stride, size_t bkg_stride,
@@ -969,6 +975,9 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
   H5T_class_t class;
   size_t tid_size;
 
+  if (hdf5_global_verbosity > 1)
+    Rprintf ("Processing attribute %s\n",attrName);
+
   if ((aid = H5Aopen_name (loc_id, attrName)) < 0)
     errorcall (ainfo->call, "could not open attribute `%s'", attrName);
 
@@ -994,6 +1003,9 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
       errorcall (ainfo->call,
 		 "could not get rank of attribute space `%s'",
 		 attrName);
+
+    if (hdf5_global_verbosity > 1) 
+      Rprintf ("attribute %s has rank %d \n",attrName,rank);    
     {
       hsize_t dims[rank];
 
@@ -1069,6 +1081,8 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
     errorcall (ainfo->call, "unable to close attribute `%s' type", attrName);
   if (H5Aclose (aid) < 0)
     errorcall (ainfo->call, "unable to close attribute `%s'", attrName);
+  if (hdf5_global_verbosity > 1) 
+    Rprintf ("Done processing attribute %s\n",attrName);
   return 0;
 }
 
@@ -1090,9 +1104,10 @@ static herr_t
 hdf5_process_object (hid_t id, const char *name, void *client_data)
 {
   struct hdf5_iterate_info *iinfo = client_data;
-
   H5G_stat_t statbuf;
 
+  if (hdf5_global_verbosity > 0)
+    Rprintf ("Processing object: %s ...",name);
   if (H5Gget_objinfo (id, name, 1, &statbuf) < 0)
     errorcall (iinfo->call, "Cannot query object `%s'", name);
 
@@ -1101,6 +1116,8 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
       SEXP l;
       hid_t gid = H5Gopen (id, name);
 
+      if (hdf5_global_verbosity > 0)
+        Rprintf ("... which is a Group \n");
       if (gid < 0)
 	errorcall (iinfo->call, "unable to open group `%s'", name);
 
@@ -1112,6 +1129,8 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
 
       if (H5Gclose (gid) < 0)
 	errorcall (iinfo->call, "unable to close group");
+      if (hdf5_global_verbosity > 0)
+        Rprintf ("... Done group %s \n",name);      
     }
   else if (statbuf.type == H5G_DATASET)
     {
@@ -1120,12 +1139,21 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
       SEXPTYPE type = NILSXP;
       /*H5T_class_t class;*/
 
+      if (hdf5_global_verbosity > 0)
+        Rprintf ("... its a dataset...");
+
       if ((dataset = H5Dopen (id, name)) < 0)
 	errorcall (iinfo->call, "unable to load dataset `%s'", name);
 
+      if (hdf5_global_verbosity > 1)
+        Rprintf ("Dataset has ID%d\n",dataset);
+
       if ((tid = H5Dget_type (dataset)) < 0)
 	errorcall (iinfo->call, "unable to get dataset type");
 
+      if (hdf5_global_verbosity > 1)
+        Rprintf ("Dataset has tid %d\n",tid);
+
       switch (H5Tget_class (tid))
 	{
 	case H5T_INTEGER:
@@ -1151,45 +1179,75 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
       if ((space = H5Dget_space (dataset)) < 0)
 	errorcall (iinfo->call, "unable to get dataset space");
 
+      if (hdf5_global_verbosity > 1)
+        Rprintf ("Dataset has space id %d\n", space);
+
       if (H5Sis_simple (space) != LTRUE)
 	errorcall (iinfo->call, "space not simple");
 
       if ((rank = H5Sget_simple_extent_ndims (space)) < 0)
 	errorcall (iinfo->call, "unable to get space rank");
 
-      {
-	hsize_t dims[rank];
-	hsize_t maxdims[rank];
+      if (hdf5_global_verbosity > 1)
+        Rprintf ("Dataset has rank %d\n",rank);
 
-	if (H5Sget_simple_extent_dims (space, dims, maxdims) < 0)
-	  errorcall (iinfo->call, "unable to get space extent");
+      if (rank == 0)
+        warningcall (iinfo->call, "Ignoring dataset %s as it has rank 0", name);
+      else
+        {
+          hsize_t dims[rank];
+          hsize_t maxdims[rank];
+          
+          if (H5Sget_simple_extent_dims (space, dims, maxdims) < 0)
+            errorcall (iinfo->call, "unable to get space extent");
 
-	if (type == VECSXP && rank == 1)
-	  {
-	    unsigned colcount = H5Tget_nmembers (tid), ci;
-	    SEXP vec;
-	    size_t size = H5Tget_size (tid);
-	    unsigned ri, rowcount = dims[0];
-	    char buf[rowcount][size];
-	    hid_t rtid = make_sexp_ref_type (iinfo->call);
-	    SEXP names;
+          if (hdf5_global_verbosity > 1)
+            {
+              hsize_t irank;
 
-	    if (H5Dread (dataset, tid, space, space, H5P_DEFAULT,
-			 buf) < 0)
-	      errorcall (iinfo->call, "can't read compound data vector");
+              Rprintf ("Dataset has dims/maxdims:");
+              for (irank = 0; irank < rank; irank++)
+                Rprintf (" %d ",dims[irank]);
+              Rprintf ("/");
+              for (irank = 0;irank < rank; irank++)
+                Rprintf (" %d ",maxdims[irank]);
+              Rprintf ("\n");
+            }
+          if (type == VECSXP && rank == 1)
+            {
+              unsigned colcount = H5Tget_nmembers (tid), ci;
+              SEXP vec;
+              size_t size = H5Tget_size (tid);
+              unsigned ri, rowcount = dims[0];
+              char buf[rowcount][size];
+              hid_t rtid = make_sexp_ref_type (iinfo->call);
+              SEXP names;
 
-	    PROTECT (vec = allocVector (VECSXP, colcount));
-	    PROTECT (names = allocVector (STRSXP, colcount));
-
-	    for (ci = 0; ci < colcount; ci++)
-	      {
-		hid_t ctid = H5Tget_member_type (tid, ci);
-		H5T_class_t class = H5Tget_class (ctid);
-		size_t csize = H5Tget_size (ctid);
-		size_t coffset = H5Tget_member_offset (tid, ci);
-		SEXPREC **rowptr = &VECTOR_ELT (vec, ci);
-		unsigned char itembuf[size]; /* for overrun */
+              if (hdf5_global_verbosity > 2) 
+                {
+                  Rprintf ("Dataset has type = VECSXP and rank 1\n");
+                  Rprintf ("Reading...\n");
+                }
 
+              if (H5Dread (dataset, tid, space, space, H5P_DEFAULT,
+                           buf) < 0)
+                errorcall (iinfo->call, "can't read compound data vector");
+              
+              if (hdf5_global_verbosity > 2)
+                Rprintf ("....done\n");
+              
+              PROTECT (vec = allocVector (VECSXP, colcount));
+              PROTECT (names = allocVector (STRSXP, colcount));
+              
+              for (ci = 0; ci < colcount; ci++)
+                {
+                  hid_t ctid = H5Tget_member_type (tid, ci);
+                  H5T_class_t class = H5Tget_class (ctid);
+                  size_t csize = H5Tget_size (ctid);
+                  size_t coffset = H5Tget_member_offset (tid, ci);
+                  SEXPREC **rowptr = &VECTOR_ELT (vec, ci);
+                  unsigned char itembuf[size]; /* for overrun */
+                  
 #define VECLOOP(vectype, vecref, dtid) \
   { \
     size_t dsize = H5Tget_size (dtid); \
@@ -1202,63 +1260,105 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
       } \
   }
 
-		{
-		  char *colname = H5Tget_member_name (tid, ci);
+                  {
+                    char *colname = H5Tget_member_name (tid, ci);
+                    
+                    if (colname)
+                      SET_STRING_ELT (names, ci, mkChar (colname));
+                  }
+                  switch (class)
+                    {
+                    case H5T_INTEGER:
+                      {
+                        SEXPTYPE type = (csize == 1) ? LGLSXP : INTSXP;
+                        
+                        *rowptr = allocVector (type, rowcount);
+                        VECLOOP (type, INTEGER, H5T_NATIVE_INT);
+                      }
+                      break;
+                    case H5T_FLOAT:
+                      *rowptr = allocVector (REALSXP, rowcount);
+                      VECLOOP (REALSXP, REAL, H5T_NATIVE_DOUBLE);
+                      break;
+                    case H5T_STRING:
+                      *rowptr = allocVector (STRSXP, rowcount);
+                      VECLOOP (STRSXP, STRING_PTR, rtid);
+                      break;
+                    default:
+                      errorcall (iinfo->call, "can't handle hdf class %d",
+                                 class);
+                    }
+                }
+              load_rownames_dataset_attribute (iinfo->call, dataset, vec);
+              setAttrib (vec, R_NamesSymbol, names);
+              UNPROTECT (1);
+              setAttrib (vec, R_ClassSymbol, mkString ("data.frame"));
+              iinfo->add (iinfo, name, vec);
+              hdf5_load_attributes (iinfo->call, dataset, vec, name);
+              UNPROTECT (1);
+              if (H5Tclose (rtid) < 0)
+                errorcall (iinfo->call, "could not close reference type");
+            }
+          else
+            {
+              SEXP obj;
+              
+              if (rank == 1)
+                {
+                  if (hdf5_global_verbosity > 2) 
+                    Rprintf ("Allocating vector with rank=%d dim=%d\n",
+                             rank,dims[0]);
+                  PROTECT (obj = allocVector (type, dims[0]));
+                }
+              else if (rank == 2)
+                {
+                  if (hdf5_global_verbosity > 2) 
+                    Rprintf ("Allocating matrix with rank=%d dim=%d %d\n",
+                             rank,dims[0],dims[1]);
+                  PROTECT (obj = allocMatrix (type, dims[0], dims[1]));
+                }
+              else
+                {
+                  SEXP dimsvector;
+                  hsize_t irank;
 
-		  if (colname)
-		    SET_STRING_ELT (names, ci, mkChar (colname));
-		}
-		switch (class) {
-		  case H5T_INTEGER:
-		    {
-		      SEXPTYPE type = (csize == 1) ? LGLSXP : INTSXP;
+                  if (hdf5_global_verbosity > 2) 
+                    Rprintf ("Allocating array with rank=%d dims:", rank);
 
-		      *rowptr = allocVector (type, rowcount);
-		      VECLOOP (type, INTEGER, H5T_NATIVE_INT);
-		    }
-		    break;
-		  case H5T_FLOAT:
-		    *rowptr = allocVector (REALSXP, rowcount);
-		    VECLOOP (REALSXP, REAL, H5T_NATIVE_DOUBLE);
-		    break;
-		  case H5T_STRING:
-		    *rowptr = allocVector (STRSXP, rowcount);
-		    VECLOOP (STRSXP, STRING_PTR, rtid);
-		    break;
-		  default:
-		    errorcall (iinfo->call, "can't handle hdf class %d",
-			       class);
-		  }
-	      }
-	    load_rownames_dataset_attribute (iinfo->call, dataset, vec);
-	    setAttrib (vec, R_NamesSymbol, names);
-	    UNPROTECT (1);
-	    setAttrib (vec, R_ClassSymbol, mkString ("data.frame"));
-	    iinfo->add (iinfo, name, vec);
-	    hdf5_load_attributes (iinfo->call, dataset, vec, name);
-	    UNPROTECT (1);
-	    if (H5Tclose (rtid) < 0)
-	      errorcall (iinfo->call, "could not close reference type");
-	  }
-	else
-	  {
-	    SEXP obj;
-
-	    PROTECT (obj = ((rank == 1)
-			    ? allocVector (type, dims[0])
-			    : allocMatrix (type, dims[0], dims[1])));
-	    vector_io (iinfo->call, LFALSE, dataset, space, obj);
-	    iinfo->add (iinfo, name, obj);
-	    hdf5_load_attributes (iinfo->call, dataset, obj, name);
-	    UNPROTECT (1);
-	  }
-      }
+                  PROTECT (dimsvector = allocVector (INTSXP, rank));
+                  for (irank = 0; irank < rank; irank++)
+                    {
+                      if (hdf5_global_verbosity > 1)
+                        Rprintf(" %d ", dims[irank]);
+                      INTEGER(dimsvector)[irank] = dims[irank];
+                    }
+                  if (hdf5_global_verbosity > 2)
+                    Rprintf ("\n");
+                  PROTECT (obj = allocArray (type, dimsvector)); 
+                  UNPROTECT (2);
+                  PROTECT (obj);
+                }
+              vector_io (iinfo->call, LFALSE, dataset, space, obj);
+              iinfo->add (iinfo, name, obj);
+              if (hdf5_global_verbosity > 2)
+                {
+                  Rprintf ("Rank > 1 or not VECSXP\n");
+                  Rprintf ("Calling  hdf5_load_attributes \n");
+                }
+              hdf5_load_attributes (iinfo->call, dataset, obj, name);
+              if (hdf5_global_verbosity > 2) 
+                Rprintf ("back from  hdf5_load_attributes \n");
+              UNPROTECT (1);
+            }
+        }
       if (H5Sclose (space) < 0)
 	errorcall (iinfo->call, "unable to close dataspace");
       if (H5Tclose (tid) < 0)
 	errorcall (iinfo->call, "unable to close datatype");
       if (H5Dclose (dataset) < 0)
 	errorcall (iinfo->call, "unable to close dataset");
+      if (hdf5_global_verbosity > 0)
+        Rprintf ("...Finished dataset \n");
     }
   else
     errorcall (iinfo->call, "no support for HDF object type: %d",
@@ -1306,8 +1406,13 @@ SEXP do_hdf5load (SEXP args)
   path = CHAR(STRING_ELT (CAR(args), 0));
   restore_syms = INTEGER (CADR(args))[0];
 
-  H5dont_atexit ();
+  hdf5_global_verbosity = INTEGER (CADDR(args))[0];
+  if (hdf5_global_verbosity > 2) 
+    Rprintf ("hdf5_global_verbosity=%d load=%d\n",
+             hdf5_global_verbosity, restore_syms);
 
+  H5dont_atexit ();
+  
   if ((fid = H5Fopen (path, H5F_ACC_RDONLY, H5P_DEFAULT)) < 0)
     errorcall (call, "unable to open HDF file: %s", path);
 
